/**
 * Convert number of bytes into human readable format
 *
 * @param integer bytes     Number of bytes to convert
 * @param integer precision Number of digits after the decimal separator
 * @return string
 */
function bytesToSize(bytes, precision) {
  var kilobyte = 1024;
  var megabyte = kilobyte * 1024;
  var gigabyte = megabyte * 1024;
  var terabyte = gigabyte * 1024;

  var sign = '';
  if (bytes < 0) {
    bytes = -bytes;
    sign = '-';
  }

  if (bytes >= 0 && bytes < kilobyte) {
    return sign + bytes + ' B';
  }
  if (bytes < megabyte) {
    return sign + (bytes / kilobyte).toFixed(precision) + ' KB';
  }
  if (bytes < gigabyte) {
    return sign + (bytes / megabyte).toFixed(precision) + ' MB';
  }
  if (bytes < terabyte) {
    return sign + (bytes / gigabyte).toFixed(precision) + ' GB';
  }
  return sign + (bytes / terabyte).toFixed(precision) + ' TB';
}
